<?php
/*****************************************************************************
 * ManageUsers.php
 * Contains the controller with the list of users.
 *
 * Kabuki CMS (C) 2013-2015, Aaron van Geffen
 *****************************************************************************/

class ManageUsers extends HTMLController
{
	public function __construct()
	{
		// Ensure it's just admins at this point.
		if (!Registry::get('user')->isAdmin())
			throw new NotAllowedException();

		Session::resetSessionToken();

		$options = [
			'form' => [
				'action' => BASEURL . '/edituser/',
				'method' => 'get',
				'class' => 'col-md-6 text-end',
				'buttons' => [
					'add' => [
						'type' => 'submit',
						'caption' => 'Add new user',
					],
				],
			],
			'columns' => [
				'id_user' => [
					'value' => 'id_user',
					'header' => 'ID',
					'is_sortable' => true,
				],
				'surname' => [
					'header' => 'Last name',
					'is_sortable' => true,
					'link' => BASEURL . '/edituser/?id={ID_USER}',
					'value' => 'surname',
				],
				'first_name' => [
					'header' => 'First name',
					'is_sortable' => true,
					'link' => BASEURL . '/edituser/?id={ID_USER}',
					'value' => 'first_name',
				],
				'slug' => [
					'header' => 'Slug',
					'is_sortable' => true,
					'link' => BASEURL . '/edituser/?id={ID_USER}',
					'value' => 'slug',
				],
				'emailaddress' => [
					'value' => 'emailaddress',
					'header' => 'Email address',
					'is_sortable' => true,
				],
				'last_action_time' => [
					'format' => [
						'type' => 'timestamp',
						'pattern' => 'long',
						'value' => 'last_action_time',
					],
					'header' => 'Last activity',
					'is_sortable' => true,
				],
				'ip_address' => [
					'is_sortable' => true,
					'value' => 'ip_address',
					'header' => 'IP address',
				],
				'is_admin' => [
					'is_sortable' => true,
					'header' => 'Admin?',
					'format' => fn($row) => $row['is_admin'] ? 'yes' : 'no',
				],
			],
			'default_sort_order' => 'id_user',
			'default_sort_direction' => 'down',
			'start' => $_GET['start'] ?? 0,
			'sort_order' => $_GET['order'] ?? '',
			'sort_direction' => $_GET['dir'] ?? '',
			'title' => 'Manage users',
			'no_items_label' => 'No users meet the requirements of the current filter.',
			'items_per_page' => 30,
			'index_class' => 'col-md-6',
			'base_url' => BASEURL . '/manageusers/',
			'get_data' => 'Member::getOffset',
			'get_count' => 'Member::getCount',
		];

		$table = new GenericTable($options);
		parent::__construct('User management - Page ' . $table->getCurrentPage() .' - ' . SITE_TITLE);
		$this->page->adopt(new TabularData($table));
	}
}
